#include "client.hpp"
#include "definitions.hpp"
#include "server.hpp"
#include <cstdio>
#include <cstring>
#include <unistd.h>

Client::Client() : sock(), serverAddr() {
  sock = socket(AF_INET, SOCK_STREAM, 0);
  serverAddr.sin_family = AF_INET;
  serverAddr.sin_port = htons(DEFAULT_PORT);
  inet_aton(DEFAULT_IP, &serverAddr.sin_addr);
  int rc = connect(sock, (struct sockaddr *)&serverAddr, sizeof(serverAddr));
  if (rc < 0) {
    perror("connect() failed");
    return;
  }
}

Client::Client(const std::string &addr, const int &port) {
  sock = socket(AF_INET, SOCK_STREAM, 0);
  serverAddr.sin_family = AF_INET;
  serverAddr.sin_port = htons(port);
  inet_aton(addr.c_str(), &serverAddr.sin_addr);
}

void Client::connectToServer() {
  int rc = connect(sock, (struct sockaddr *)&serverAddr, sizeof(serverAddr));
  if (rc < 0) {
    perror("connect() failed");
    return;
  }
}

void Client::disconnectFromServer() { close(sock); }

uint16_t Client::sendMessage(const char *msg) {
  return send(sock, msg, strlen(msg) + 1, 0);
}

wordle_de::Message Client::receiveMessage() {
  int nbytesrecv = recv(sock, inputBuffer, INPUT_BUFFER_SIZE, 0);
  wordle_de::Message msg;
  if (nbytesrecv <= 0) {
    if (nbytesrecv == 0) {
      disconnectFromServer();
      return msg;
    } else {
      perror("recv() failed");
    }
    close(sock);
    return msg;
  }
  msg.ParseFromString(inputBuffer);
  bzero(&inputBuffer, INPUT_BUFFER_SIZE);
  return msg;
}
