#ifndef _STATE_H
#define _STATE_H

#include "definitions.hpp"
#include "wordle-de.pb.h"
#include <array>
#include <functional>
#include <mutex>
#include <string>
#include <vector>

class GameState {
public:
  enum matchType { NO_MATCH, MATCH, EXACT_MATCH };
  typedef std::array<matchType, WORD_LENGTH> guessResultT;
  struct guess {
    std::string guess;
    guessResultT result;
  };

private:
  std::string word;
  uint8_t tries;
  std::vector<std::string> guesses;

public:
  GameState();
  GameState(const std::string &word);
  void setWord(const std::string &);
  guessResultT guess(const std::string &);
  std::string serialize(bool);
};

class Engine {
public:
  typedef std::function<void(const std::string &)> Callback;

private:
  std::string currentWord;
  std::vector<std::string> pastWords;
  std::string wordFile;
  bool started = false;
  std::mutex mut;
  std::vector<Callback> callbacks;

public:
  static void createInstance();
  static void createInstance(const std::string &);
  static Engine *getInstance();
  static void destroyInstance();

  void start(unsigned int);
  void setNewWord();
  void setWord(const std::string &);
  std::string getWord();
  void registerNewWordCallback(Callback);

private:
  Engine() : wordFile(DEFAULT_FILENAME), mut() {}
  Engine(const std::string &f) : wordFile(f) {}
  unsigned int getRandomLine(unsigned int);
  static Engine *instance;
};
#endif // _STATE_H
